import os from 'os';
import { Server } from '..';
import { Route } from '../../class';

export default class Root extends Route {
  constructor(server: Server) {
    super(server, { path: '/', deprecated: false, maintenance: false });
  }

  public bind() {
    this.router.get('/', async (_req, res) => {
      try {
        const date = new Date();
        date.setSeconds(-process.uptime());
        const accounts = await this.server.client.db.Account.find().lean().exec();
        const administrators = accounts.filter((account) => account.root === true).length;
        const technicians = accounts.filter((account) => account?.permissions?.director === true).length;
        const staff = accounts.filter((account) => account?.permissions?.staff === true).length;
        const response = {
          csd: {
            nodeVersion: process.version,
            uptime: process.uptime(),
          },
          server: {
            userCount: accounts.length,
            administratorCount: administrators,
            technicianCount: technicians,
            staffCount: staff,
            stats: {
              uptime: os.uptime(),
              loadAverages: os.loadavg(),
              hostname: os.hostname(),
              ipv4Address: os.networkInterfaces().enp0s3.filter((r) => r.family === 'IPv4')[0].address,
              operatingSystem: {
                platform: os.platform(),
                release: os.release(),
              },
              cpu: {
                model: os.cpus()[0].model,
                clockSpeed: os.cpus()[0].speed / 1000,
                coreCount: os.cpus().length,
              },
            },
          },
        };
        res.status(200).json({ code: this.constants.codes.SUCCESS, message: response });
      } catch (error) {
        this.handleError(error, res);
      }
    });
  }
}
