/* eslint-disable no-case-declarations */
/* eslint-disable consistent-return */
import net from 'net';
import crypto from 'crypto';
import { promises as fs } from 'fs';
import { Client } from '../class';
import { dataConversion } from '../functions';

export default class CSCLI {
  public client: Client;

  public server: net.Server;

  #hmac: string;

  constructor(client: Client) {
    this.client = client;
    this.loadKeys();
    this.server = net.createServer((socket) => {
      socket.on('data', async (data) => {
        try {
          await this.handle(socket, data);
        } catch (err) {
          await this.client.util.handleError(err);
          socket.destroy();
        }
      });
    });
    this.init();
  }

  public async handle(socket: net.Socket, data: Buffer) {
    const args = data.toString().trim().split('$');
    const verification = this.verifyConnection(args[1], args[0]);
    if (!verification) {
      socket.write('UNAUTHORIZED TO EXECUTE ON THIS SERVER\n');
      return socket.destroy();
    }
    const parsed: { Username: string, Type: string, Message?: string, HMAC: string } = JSON.parse(args[0]);
    // FINISH VERIFICATION CHECKS
    switch (parsed.Type) {
      case 'lock':
        await this.client.util.accounts.lock(parsed.Username, this.client.user.id, { reason: parsed.Message });
        break;
      case 'killpid':
        await this.client.util.exec(`kill -9 ${parsed.Message}`);
        break;
      case 'ram':
        const memoryConversion = dataConversion(Number(await this.client.util.exec(`memory ${parsed.Username}`)) * 1000);
        socket.write(`${memoryConversion}\n`);
        socket.destroy();
        break;
      case 'storage':
        const res = await this.client.redis.get(`storage-${parsed.Username}`) ? dataConversion(Number(await this.client.redis.get(`storage-${parsed.Username}`))) : 'N/A';
        socket.write(`${res}\n`);
        socket.destroy();
        break;
      case 'processcount':
        const processCount = await this.client.util.exec(`ps -U ${parsed.Username} -u ${parsed.Username} u | wc -l`);
        socket.write(`${processCount}\n`);
        socket.destroy();
        break;
      case 'sshlogins':
        const sshLogins = await this.client.util.exec(`who | grep ${parsed.Username} | wc -l`);
        socket.write(`${sshLogins}\n`);
        socket.destroy();
        break;
      case 'ramlimit':
        const account = await this.client.db.Account.findOne({ username: parsed.Username }).lean().exec();
        const tier = await this.client.db.Tier.findOne({ id: account.tier }).lean().exec();
        socket.write(`${tier.resourceLimits.ram}\n`);
        socket.destroy();
        break;
      default:
        socket.destroy();
        break;
    }
  }

  public verifyConnection(key: string, data: any): boolean {
    const hmac = crypto.createHmac('sha256', this.#hmac);
    hmac.update(data);
    const computed = hmac.digest('hex');
    if (computed === key) return true;
    return false;
  }

  public async loadKeys() {
    const key = await fs.readFile('/etc/cscli.conf', { encoding: 'utf8' });
    this.#hmac = key.toString().trim();
  }

  public init() {
    this.server.on('error', (err) => {
      this.client.util.handleError(err);
    });
    this.server.listen(8124, () => {
      this.client.signale.success('TCP socket is now listening for connections.');
    });
  }
}
